//
// File: AirportSocket.h
// Created by: Kristian Müller <Kristian-M@Kristian-M.de>
// Created on: Wed Oct  1 16:28:32 2003
//

#include "../include/AirportSocket.h"
// #include <iostream.h> //debugging

AirportSocket::AirportSocket(char *sInHostname) : IPv4Socket(sInHostname, 192){
	GVar aGVar;

	setPort(aGVar.getDefaultPort());
	bModemConnected = false;
//	aLocalStatusData = NULL;
}


AirportSocket::~AirportSocket(){
	// nothing to destruct jet
//	if (aLocalStatusData != NULL)
//		delete aLocalStatusData;
}

int AirportSocket::connectModem() {
	unsigned char sPacket[116]	= {0};
	sPacket[0] = 0x07;
	writeToSocket(sPacket,116);
	bModemConnected = true;
	return 1;
}

int AirportSocket::disconnectModem(){
	unsigned char sPacket[116]	= {0};
	sPacket[0] = 0x06;
	writeToSocket(sPacket,116);
	bModemConnected = false;
	return 1;
}

// OSU-NMS conversion routine
// Mostly from AirportBaseStationHangup Utility Copyright (C) Jon Sevy
// (jsevy@mcs.drexel.edu)
//
// NOTE: ntohs() and ntohl() seem to work in lieu of the additive loop
long int AirportSocket::convertOsuNms2Int(void *aInBuffer, int iInLenght){
	signed char *sBuffer = (signed char *)aInBuffer;
	long int iReturnValue = 0;

	if (iInLenght == 2) {
		iReturnValue	 = ntohs(*(short *)aInBuffer);	
	} else if (iInLenght == 4) {
		iReturnValue	 = ntohs(*(long *)aInBuffer);
	} else {
	for (int iCounter = 0; iCounter < iInLenght; iCounter++) {
			iReturnValue =+ (iReturnValue*256) + ((sBuffer[iCounter] < 0)? (sBuffer[iCounter]+256) : sBuffer[iCounter]);
		}
	}
	return iReturnValue;
}

// OSU-NMS packet format
// 1) 2 bytes packet type 
// 2) 2 bytes element length (exclusive of this; whole element = 2+2+N)
// 	a) 2 bytes tag name
// 	b) N bytes value
// 3) goto #2
AirportSocket::tOsuNmsElement *AirportSocket::osunms_element_get(struct tOsuNmsElement *aInElement, void *aInBuffer, int iInSize, int iInTag){
	signed char *sPacket	= (signed char *)aInBuffer;
	int iLength;

	sPacket	+= 2; /* skip first two bytes; */
	while ((sPacket - (signed char *)aInBuffer) < (iInSize-4)) {
		iLength	= ntohs(*(short *)sPacket);
		sPacket += 2;
		if (ntohs(*(short int *)sPacket) == iInTag) {
			aInElement->sValue	= sPacket+2;
			aInElement->iLength	= (iLength-2);
			aInElement->iTag	 = iInTag;
			return aInElement;
		}
		sPacket	+= iLength;
	}
	return NULL;
}


StatusData *AirportSocket::getStatus(StatusData **aLocalStatusData){
	unsigned char sPacket[128]	= {0};
	char baudtag[]	= {0x03,0x15};
	char timetag[]	= {0x03,0x16};
	char rmaintag[]	= {0x03,0x19};
	int total, hrs, min, sec;
	char *sOnlineTime = NULL;
	int iOnlineStatus;
	struct tOsuNmsElement aOsuNmsElement;
	
	sPacket[0] = 0x08;
	sPacket[1] = 0x01;
	sPacket[2] = 0x03;
	sPacket[3] = 0x10;
	
	if (writeToSocket(sPacket, 116) == -1) return NULL;
	if (readFromSocket(sPacket, 128) == -1) return NULL;

	*aLocalStatusData = new StatusData();

	switch (sPacket[6]) {
		case 4: /* connected */
			iOnlineStatus = 1;

			if (osunms_element_get(&aOsuNmsElement, sPacket, 128, convertOsuNms2Int(timetag,2)) != NULL) {
				total	= convertOsuNms2Int(aOsuNmsElement.sValue,aOsuNmsElement.iLength);
				hrs		= total / 3600;
        min		= (total / 60) % 60;
        sec		= total % 60;
				sOnlineTime = new char[12];
			  sprintf(sOnlineTime, "%.2d:%.2d:%.2d", hrs, min, sec);	
			}
			break;
		case 5: /* modem temporarily disabled */
			iOnlineStatus = 4;

			if (osunms_element_get(&aOsuNmsElement, sPacket, 128, convertOsuNms2Int(rmaintag,2)) != NULL) {
				total	= convertOsuNms2Int(aOsuNmsElement.sValue, aOsuNmsElement.iLength);
				hrs		= total / 3600;
				min		= (total / 60) % 60;
				sec		= total % 60;
				sOnlineTime = new char[12];
				sprintf(sOnlineTime, "%.2d:%.2d:%.2d", hrs, min, sec);	
			}

			break;
		case 2:
		case 3: /* connecting */
			iOnlineStatus = 2;
			sOnlineTime = new char[12];
			sprintf(sOnlineTime, "--:--:--");	
			break;
		case 1:
		default: /* disconnected (not disabled) */
			iOnlineStatus = 3;
			sOnlineTime = new char[12];
			sprintf(sOnlineTime, "--:--:--");	
	}

	(*aLocalStatusData)->setOnlineStatus(iOnlineStatus);
	if (sOnlineTime != NULL)
		(*aLocalStatusData)->setOnlineTime(sOnlineTime);
	return (*aLocalStatusData);
}
